@@ -1160,6 +1160,7 @@ cli_cmd_gsync_set_parse (const char **words, int wordcount, dict_t **options)
          * volume geo-replication [$m [$s]] status
          * volume geo-replication [$m] $s config [[!]$opt [$val]]
          * volume geo-replication $m $s start|stop
+         * volume geo-replication $m [$s] log-rotate
          */
 
         if (wordcount < 3)
@@ -1219,6 +1220,11 @@ cli_cmd_gsync_set_parse (const char **words, int wordcount, dict_t **options)
 
                 if (!masteri || !slavei)
                         goto out;
+        } else if (strcmp(words[cmdi], "log-rotate") == 0) {
+                type = GF_GSYNC_OPTION_TYPE_ROTATE;
+
+                if (slavei && !masteri)
+                        goto out;
         } else
                 goto out;
 
