@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU Affero General Public License as published
+  it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
+  General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License
+  You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -132,7 +132,9 @@ cli_cmd_process (struct cli_state *state, int argc, char **argv)
 	if ( strcmp (word->word,"help")==0 )
 		goto callback;
 
-        ret = cli_cmd_await_connected (cli_cmd_needs_connection (word));
+        state->await_connected = cli_cmd_needs_connection (word);
+
+        ret = cli_cmd_await_connected (state->await_connected);
         if (ret) {
                 cli_out ("Connection failed. Please check if gluster "
                           "daemon is operational.");
