@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU Affero General Public License as published
+  it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
+  General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License
+  You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -83,11 +83,11 @@ static char argp_doc[] = "";
 const char *argp_program_version = ""                                 \
         PACKAGE_NAME" "PACKAGE_VERSION" built on "__DATE__" "__TIME__ \
         "\nRepository revision: " GLUSTERFS_REPOSITORY_REVISION "\n"  \
-        "Copyright (c) 2006-2010 Gluster Inc. "                       \
+        "Copyright (c) 2006-2011 Gluster Inc. "                       \
         "<http://www.gluster.com>\n"                                  \
         "GlusterFS comes with ABSOLUTELY NO WARRANTY.\n"              \
         "You may redistribute copies of GlusterFS under the terms of "\
-        "the GNU Affero General Public License.";
+        "the GNU General Public License.";
 
 const char *argp_program_bug_address = "<" PACKAGE_BUGREPORT ">";
 
@@ -352,6 +352,12 @@ cli_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
         {
                 gf_log (this->name, GF_LOG_TRACE, "got RPC_CLNT_DISCONNECT");
                 connected = 0;
+                if (!global_state->prompt && global_state->await_connected) {
+                        ret = 1;
+                        cli_out ("Connection failed. Please check if gluster "
+                                  "daemon is operational.");
+                        exit (ret);
+                }
                 break;
         }
 
