@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU Affero General Public License as published
+  it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
+  General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License
+  You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -76,6 +76,22 @@ gf_proc_dump_call_frame (call_frame_t *call_frame, const char *key_buf,...)
                 gf_proc_dump_build_key(key, prefix,"parent");
                 gf_proc_dump_write(key, "%s", my_frame.parent->this->name);
         }
+        if (my_frame.wind_from) {
+                gf_proc_dump_build_key(key, prefix, "wind_from");
+                gf_proc_dump_write(key, "%s", my_frame.wind_from);
+        }
+        if (my_frame.wind_to) {
+                gf_proc_dump_build_key(key, prefix, "wind_to");
+                gf_proc_dump_write(key, "%s", my_frame.wind_to);
+        }
+        if (my_frame.unwind_from) {
+                gf_proc_dump_build_key(key, prefix, "unwind_from");
+                gf_proc_dump_write(key, "%s", my_frame.unwind_from);
+        }
+        if (my_frame.unwind_to) {
+                gf_proc_dump_build_key(key, prefix, "unwind_to");
+                gf_proc_dump_write(key, "%s", my_frame.unwind_to);
+        }
 }
 
 
