@@ -97,9 +97,15 @@ synctask_wake (struct synctask *task)
 
 
 void
-synctask_wrap (struct synctask *task)
+synctask_wrap (struct synctask *old_task)
 {
         int              ret;
+        struct synctask *task = NULL;
+
+        /* Do not trust the pointer received. It may be
+           wrong and can lead to crashes. */
+
+        task = synctask_get ();
 
         ret = task->syncfn (task->opaque);
         task->synccbk (ret, task->opaque);
@@ -332,6 +338,7 @@ entry_copy (gf_dirent_t *source)
         sink->d_off = source->d_off;
         sink->d_ino = source->d_ino;
         sink->d_type = source->d_type;
+        uuid_copy (sink->d_stat.ia_gfid, source->d_stat.ia_gfid);
 
         return sink;
 }
