@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU Affero General Public License as published
+  it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
+  General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License
+  You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -78,9 +78,19 @@ typedef int32_t (*event_notify_fn_t) (xlator_t *this, int32_t event, void *data,
 struct _loc {
         const char *path;
         const char *name;
-        ino_t       ino;
         inode_t    *inode;
         inode_t    *parent;
+        /* Currently all location based operations are through 'gfid' of inode.
+         * But the 'inode->gfid' only gets set in higher most layer (as in,
+         * 'fuse', 'protocol/server', or 'nfs/server'). So if translators want
+         * to send fops on a inode before the 'inode->gfid' is set, they have to
+         * make use of below 'gfid' fields
+         */
+        uuid_t      gfid;
+        uuid_t      pargfid;
+
+        /* ideally, should not be used */
+        ino_t       ino;
 };
 
 
@@ -776,6 +786,7 @@ typedef struct volume_options {
         char                *value[ZR_OPTION_MAX_ARRAY_SIZE];
         /* If specified, will check for one of
            the value from this array */
+        char                *default_value;
         char                *description; /* about the key */
 } volume_option_t;
 
@@ -871,7 +882,8 @@ int _volume_option_value_validate_attacherr (xlator_t *xl,
                                data_pair_t *pair, 
                                volume_option_t *opt, 
                                char **op_errstr);
-
-
-
+int32_t xlator_volopt_dynload (char *xlator_type, void **dl_handle,
+                    volume_opt_list_t *vol_opt_handle);
+int xlator_get_volopt_info (struct list_head *opt_list, char *key,
+                            char **def_val, char **descr);
 #endif /* _XLATOR_H */
