@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU Affero General Public License as published
+  it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
+  General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License
+  You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -346,7 +346,8 @@ rpcsvc_auth_unix_auxgids (rpcsvc_request_t *req, int *arrlen)
         if ((!req) || (!arrlen))
                 return NULL;
 
-        if ((req->cred.flavour != AUTH_UNIX) ||
+        /* In case of AUTH_NULL auxgids are not used */
+        if ((req->cred.flavour != AUTH_UNIX) &&
             (req->cred.flavour != AUTH_GLUSTERFS)) {
                 gf_log ("", GF_LOG_DEBUG, "auth type not unix or glusterfs");
                 return NULL;
