@@ -515,6 +515,8 @@ afr_local_sh_cleanup (afr_local_t *local, xlator_t *this)
         if (sh->fresh_parent_dirs)
                 GF_FREE (sh->fresh_parent_dirs);
 
+        if (sh->write_needed)
+                GF_FREE (sh->write_needed);
         loc_wipe (&sh->parent_loc);
         loc_wipe (&sh->lookup_loc);
 
@@ -924,7 +926,6 @@ afr_detect_self_heal_by_lookup_status (afr_local_t *local, xlator_t *this)
             IA_ISREG (local->cont.lookup.inode->ia_type)) {
                 local->self_heal.need_data_self_heal = _gf_true;
                 local->self_heal.need_gfid_self_heal    = _gf_true;
-                local->self_heal.need_missing_entry_self_heal    = _gf_true;
                 gf_log (this->name, GF_LOG_WARNING,
                         "split brain detected during lookup of %s.",
                         local->loc.path);
@@ -1461,8 +1462,9 @@ afr_error_more_important (int32_t old_errno, int32_t new_errno)
         if (old_errno == ESTALE)
                 ret = _gf_false;
 
-        /* Nothing should overwrite ENOENT, except ESTALE */
-        else if ((old_errno == ENOENT) && (new_errno != ESTALE))
+        /* Nothing should overwrite ENOENT, except ESTALE/EIO*/
+        else if ((old_errno == ENOENT) && (new_errno != ESTALE)
+                 && (new_errno != EIO))
                 ret = _gf_false;
 
         return ret;
