@@ -51,7 +51,8 @@
 
 
 int
-afr_examine_dir_sh_unwind (call_frame_t *frame, xlator_t *this)
+afr_examine_dir_sh_unwind (call_frame_t *frame, xlator_t *this, int32_t op_ret,
+                           int32_t op_errno)
 {
         afr_local_t *local  = NULL;
 
@@ -70,15 +71,19 @@ gf_boolean_t
 __checksums_differ (uint32_t *checksum, int child_count,
                     unsigned char *child_up)
 {
-        int      ret   = _gf_false;
-        int      i     = 0;
-        uint32_t cksum = 0;
-
-        cksum = checksum[0];
+        int          ret            = _gf_false;
+        int          i              = 0;
+        uint32_t     cksum          = 0;
+        gf_boolean_t activate_check = _gf_false;
 
         for (i = 0; i < child_count; i++) {
                 if (!child_up[i])
                         continue;
+                if (_gf_false == activate_check) {
+                        cksum          = checksum[i];
+                        activate_check = _gf_true;
+                        continue;
+                }
 
                 if (cksum != checksum[i]) {
                         ret = _gf_true;
@@ -173,7 +178,7 @@ out:
                                 " forced merge option set",
                                 sh_type_str, local->loc.path);
 
-                        afr_self_heal (frame, this);
+                        afr_self_heal (frame, this, local->fd->inode);
                 } else {
                         afr_set_opendir_done (this, local->fd->inode);
 
@@ -520,7 +525,6 @@ afr_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         afr_private_t *  priv        = NULL;
         afr_local_t *    local       = NULL;
         xlator_t **      children    = NULL;
-        ino_t            inum        = 0;
         int              call_child  = 0;
         int              first_call_child  = 0;
         int              ret         = 0;
@@ -582,13 +586,6 @@ afr_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
         if (op_ret != -1) {
                 list_for_each_entry_safe (entry, tmp, &entries->list, list) {
-                        inum = afr_itransform (entry->d_ino, priv->child_count,
-                                               call_child);
-                        entry->d_ino = inum;
-                        inum  = afr_itransform (entry->d_stat.ia_ino,
-                                                priv->child_count, call_child);
-                        entry->d_stat.ia_ino = inum;
-
                         if ((local->fd->inode == local->fd->inode->table->root)
                             && !strcmp (entry->d_name, GF_REPLICATE_TRASH_DIR)) {
                                 list_del_init (&entry->list);
