@@ -54,6 +54,8 @@ afr_build_parent_loc (loc_t *parent, loc_t *child)
         char *tmp = NULL;
 
         if (!child->parent) {
+                //this should never be called with root as the child
+                GF_ASSERT (0);
                 loc_copy (parent, child);
                 return;
         }
@@ -101,11 +103,6 @@ afr_create_unwind (call_frame_t *frame, xlator_t *this)
                         unwind_buf = &local->cont.create.buf;
                 }
 
-                unwind_buf->ia_ino = local->cont.create.ino;
-
-                local->cont.create.preparent.ia_ino  = local->cont.create.parent_ino;
-                local->cont.create.postparent.ia_ino = local->cont.create.parent_ino;
-
                 AFR_STACK_UNWIND (create, main_frame,
                                   local->op_ret, local->op_errno,
                                   local->cont.create.fd,
@@ -174,11 +171,6 @@ afr_create_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         if (local->success_count == 0) {
                                 local->cont.create.buf        = *buf;
 
-                                local->cont.create.ino =
-                                        afr_itransform (buf->ia_ino,
-                                                        priv->child_count,
-                                                        child_index);
-
                                 if (priv->read_child >= 0) {
                                         afr_set_read_child (this, inode,
                                                             priv->read_child);
@@ -188,13 +180,6 @@ afr_create_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                                 }
                         }
 
-                        if (child_index == local->first_up_child) {
-                                local->cont.create.ino =
-                                        afr_itransform (buf->ia_ino,
-                                                        priv->child_count,
-                                                        local->first_up_child);
-                        }
-
                         if (child_index == local->read_child_index) {
                                 local->cont.create.read_child_buf = *buf;
                                 local->cont.create.preparent      = *preparent;
@@ -382,11 +367,6 @@ afr_mknod_unwind (call_frame_t *frame, xlator_t *this)
                         unwind_buf = &local->cont.mknod.buf;
                 }
 
-                unwind_buf->ia_ino = local->cont.mknod.ino;
-
-                local->cont.mknod.preparent.ia_ino  = local->cont.mknod.parent_ino;
-                local->cont.mknod.postparent.ia_ino = local->cont.mknod.parent_ino;
-
                 AFR_STACK_UNWIND (mknod, main_frame,
                                   local->op_ret, local->op_errno,
                                   local->cont.mknod.inode,
@@ -424,10 +404,6 @@ afr_mknod_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
                         if (local->success_count == 0){
                                 local->cont.mknod.buf   = *buf;
-                                local->cont.mknod.ino   =
-                                        afr_itransform (buf->ia_ino,
-                                                        priv->child_count,
-                                                        child_index);
 
                                 if (priv->read_child >= 0) {
                                         afr_set_read_child (this, inode,
@@ -438,13 +414,6 @@ afr_mknod_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                                 }
                         }
 
-                        if (child_index == local->first_up_child) {
-                                local->cont.mknod.ino =
-                                        afr_itransform (buf->ia_ino,
-                                                        priv->child_count,
-                                                        local->first_up_child);
-                        }
-
                         if (child_index == local->read_child_index) {
                                 local->cont.mknod.read_child_buf = *buf;
                                 local->cont.mknod.preparent      = *preparent;
@@ -625,11 +594,6 @@ afr_mkdir_unwind (call_frame_t *frame, xlator_t *this)
                         unwind_buf = &local->cont.mkdir.buf;
                 }
 
-                unwind_buf->ia_ino = local->cont.mkdir.ino;
-
-                local->cont.mkdir.preparent.ia_ino  = local->cont.mkdir.parent_ino;
-                local->cont.mkdir.postparent.ia_ino = local->cont.mkdir.parent_ino;
-
                 AFR_STACK_UNWIND (mkdir, main_frame,
                                   local->op_ret, local->op_errno,
                                   local->cont.mkdir.inode,
@@ -668,11 +632,6 @@ afr_mkdir_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         if (local->success_count == 0) {
                                 local->cont.mkdir.buf   = *buf;
 
-                                local->cont.mkdir.ino =
-                                        afr_itransform (buf->ia_ino,
-                                                        priv->child_count,
-                                                        child_index);
-
                                 if (priv->read_child >= 0) {
                                         afr_set_read_child (this, inode,
                                                             priv->read_child);
@@ -682,13 +641,6 @@ afr_mkdir_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                                 }
                         }
 
-                        if (child_index == local->first_up_child) {
-                                local->cont.mkdir.ino =
-                                        afr_itransform (buf->ia_ino,
-                                                        priv->child_count,
-                                                        local->first_up_child);
-                        }
-
                         if (child_index == local->read_child_index) {
                                 local->cont.mkdir.read_child_buf = *buf;
                                 local->cont.mkdir.preparent      = *preparent;
@@ -870,11 +822,6 @@ afr_link_unwind (call_frame_t *frame, xlator_t *this)
                         unwind_buf = &local->cont.link.buf;
                 }
 
-                unwind_buf->ia_ino = local->cont.link.ino;
-
-                local->cont.link.preparent.ia_ino  = local->cont.link.parent_ino;
-                local->cont.link.postparent.ia_ino = local->cont.link.parent_ino;
-
                 AFR_STACK_UNWIND (link, main_frame,
                                   local->op_ret, local->op_errno,
                                   local->cont.link.inode,
@@ -1100,11 +1047,6 @@ afr_symlink_unwind (call_frame_t *frame, xlator_t *this)
                         unwind_buf = &local->cont.symlink.buf;
                 }
 
-                unwind_buf->ia_ino = local->cont.symlink.ino;
-
-                local->cont.symlink.preparent.ia_ino  = local->cont.symlink.parent_ino;
-                local->cont.symlink.postparent.ia_ino = local->cont.symlink.parent_ino;
-
                 AFR_STACK_UNWIND (symlink, main_frame,
                                   local->op_ret, local->op_errno,
                                   local->cont.symlink.inode,
@@ -1142,9 +1084,6 @@ afr_symlink_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
                         if (local->success_count == 0) {
                                 local->cont.symlink.buf        = *buf;
-                                local->cont.symlink.ino =
-                                        afr_itransform (buf->ia_ino, priv->child_count,
-                                                        child_index);
 
                                 if (priv->read_child >= 0) {
                                         afr_set_read_child (this, inode,
@@ -1155,13 +1094,6 @@ afr_symlink_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                                 }
                         }
 
-                        if (child_index == local->first_up_child) {
-                                local->cont.symlink.ino =
-                                        afr_itransform (buf->ia_ino,
-                                                        priv->child_count,
-                                                        local->first_up_child);
-                        }
-
                         if (child_index == local->read_child_index) {
                                 local->cont.symlink.read_child_buf = *buf;
                                 local->cont.symlink.preparent      = *preparent;
@@ -1342,13 +1274,6 @@ afr_rename_unwind (call_frame_t *frame, xlator_t *this)
                         unwind_buf = &local->cont.rename.buf;
                 }
 
-                unwind_buf->ia_ino = local->cont.rename.ino;
-
-                local->cont.rename.preoldparent.ia_ino  = local->cont.rename.oldparent_ino;
-                local->cont.rename.postoldparent.ia_ino = local->cont.rename.oldparent_ino;
-                local->cont.rename.prenewparent.ia_ino  = local->cont.rename.newparent_ino;
-                local->cont.rename.postnewparent.ia_ino = local->cont.rename.newparent_ino;
-
                 AFR_STACK_UNWIND (rename, main_frame,
                                   local->op_ret, local->op_errno,
                                   unwind_buf,
@@ -1559,9 +1484,6 @@ afr_unlink_unwind (call_frame_t *frame, xlator_t *this)
         UNLOCK (&frame->lock);
 
         if (main_frame) {
-                local->cont.unlink.preparent.ia_ino  = local->cont.unlink.parent_ino;
-                local->cont.unlink.postparent.ia_ino = local->cont.unlink.parent_ino;
-
                 AFR_STACK_UNWIND (unlink, main_frame,
                                   local->op_ret, local->op_errno,
                                   &local->cont.unlink.preparent,
@@ -1759,9 +1681,6 @@ afr_rmdir_unwind (call_frame_t *frame, xlator_t *this)
         UNLOCK (&frame->lock);
 
         if (main_frame) {
-                local->cont.rmdir.preparent.ia_ino  = local->cont.rmdir.parent_ino;
-                local->cont.rmdir.postparent.ia_ino = local->cont.rmdir.parent_ino;
-
                 AFR_STACK_UNWIND (rmdir, main_frame,
                                   local->op_ret, local->op_errno,
                                   &local->cont.rmdir.preparent,
