@@ -277,7 +277,8 @@ __unopened_count (int child_count, unsigned int *opened_on, unsigned char *child
 
 
 int
-afr_openfd_sh_unwind (call_frame_t *frame, xlator_t *this)
+afr_openfd_sh_unwind (call_frame_t *frame, xlator_t *this, int32_t op_ret,
+                      int32_t op_errno)
 {
         afr_local_t   *local      = NULL;
         afr_private_t *priv       = NULL;
@@ -321,9 +322,6 @@ afr_openfd_sh_unwind (call_frame_t *frame, xlator_t *this)
         UNLOCK (&local->fd->lock);
 
         if (call_count == 0) {
-                gf_log (this->name, GF_LOG_WARNING,
-                        "fd not open on any subvolume %p (%s)",
-                        local->fd, local->loc.path);
                 abandon = 1;
                 goto out;
         }
@@ -418,9 +416,8 @@ afr_openfd_sh (call_frame_t *frame, xlator_t *this)
         GF_ASSERT (local->loc.path);
         /* forcibly trigger missing-entries self-heal */
 
-        local->success_count    = 1;
-        local->enoent_count     = 1;
-
+        sh->need_missing_entry_self_heal = _gf_true;
+        sh->need_gfid_self_heal = _gf_true;
         sh->data_lock_held      = _gf_true;
         sh->need_data_self_heal = _gf_true;
         sh->type                = local->fd->inode->ia_type;
@@ -434,7 +431,7 @@ afr_openfd_sh (call_frame_t *frame, xlator_t *this)
                 "path: %s, reason: Replicate up down flush, data lock is held",
                 sh_type_str, local->loc.path);
 
-        afr_self_heal (frame, this);
+        afr_self_heal (frame, this, local->fd->inode);
 
         return 0;
 }
@@ -619,9 +616,6 @@ afr_openfd_flush (call_frame_t *frame, xlator_t *this, fd_t *fd)
         UNLOCK (&local->fd->lock);
 
         if (call_count == 0) {
-                gf_log (this->name, GF_LOG_WARNING,
-                        "fd not open on any subvolume %p (%s)",
-                        fd, local->loc.path);
                 no_open = 1;
                 goto out;
         }
