@@ -34,11 +34,6 @@ typedef enum {
         AFR_LOOKUP_FAIL_MISSING_GFIDS = 2,
 } afr_lookup_flags_t;
 
-typedef enum {
-        AFR_SPLIT_BRAIN = 1,
-        AFR_ALL_FOOLS =2
-} afr_source_flags_t;
-
 int
 afr_sh_select_source (int sources[], int child_count);
 
@@ -53,6 +48,7 @@ afr_sh_print_pending_matrix (int32_t *pending_matrix[], xlator_t *this);
 
 int
 afr_build_pending_matrix (char **pending_key, int32_t **pending_matrix,
+                          unsigned char *ignorant_subvols,
                           dict_t *xattr[], afr_transaction_type type,
                           size_t child_count);
 
@@ -62,10 +58,9 @@ afr_sh_pending_to_delta (afr_private_t *priv, dict_t **xattr,
                          int child_count, afr_transaction_type type);
 
 int
-afr_mark_sources (int32_t *sources, int32_t **pending_matrix, struct iatt *bufs,
-                  int32_t child_count, afr_self_heal_type type,
-                  int32_t *valid_children, const char *xlator_name,
-                  afr_source_flags_t *flags);
+afr_mark_sources (xlator_t *this, int32_t *sources, int32_t **pending_matrix,
+                  struct iatt *bufs, afr_self_heal_type type,
+                  int32_t *success_children, int32_t *subvol_status);
 
 int
 afr_sh_delta_to_xattr (afr_private_t *priv,
@@ -83,10 +78,10 @@ afr_self_heal_type
 afr_self_heal_type_for_transaction (afr_transaction_type type);
 
 int
-afr_build_sources (xlator_t *xlator, dict_t **xattr, struct iatt *bufs,
+afr_build_sources (xlator_t *this, dict_t **xattr, struct iatt *bufs,
                    int32_t **pending_matrix, int32_t *sources,
                    int32_t *success_children, afr_transaction_type type,
-                   afr_source_flags_t *flags);
+                   int32_t *subvol_status, gf_boolean_t ignore_ignorant);
 void
 afr_sh_common_reset (afr_self_heal_t *sh, unsigned int child_count);
 int
@@ -118,4 +113,9 @@ int
 afr_impunge_frame_create (call_frame_t *frame, xlator_t *this,
                           int active_source, int ret_child, mode_t entry_mode,
                           call_frame_t **impunge_frame);
+void
+afr_sh_reset (call_frame_t *frame, xlator_t *this);
+int
+afr_get_no_xattr_dir_read_child (xlator_t *this, int32_t *success_children,
+                                 struct iatt *bufs);
 #endif /* __AFR_SELF_HEAL_COMMON_H__ */
