@@ -210,6 +210,7 @@ typedef struct {
 
         /* private data for the particular self-heal algorithm */
         void *private;
+        unsigned char *write_needed;
 
         int (*flush_self_heal_cbk) (call_frame_t *frame, xlator_t *this);
 
@@ -221,6 +222,10 @@ typedef struct {
         call_frame_t *sh_frame;
 } afr_self_heal_t;
 
+typedef enum {
+        SPLIT_BRAIN = 1,
+        ALL_FOOLS = 2
+} afr_subvol_status_t;
 
 typedef enum {
         AFR_DATA_TRANSACTION,          /* truncate, write, ... */
