@@ -96,6 +96,7 @@ struct dht_local {
         char                     need_selfheal;
         int                      file_count;
         int                      dir_count;
+        int                      fop_succeeded;
         call_frame_t            *main_frame;
         struct {
                 fop_mknod_cbk_t  linkfile_cbk;
@@ -181,7 +182,6 @@ struct dht_disk_layout {
 };
 typedef struct dht_disk_layout dht_disk_layout_t;
 
-#define WIPE(statp) do { typeof(*statp) z = {0,}; if (statp) *statp = z; } while (0)
 
 #define ENTRY_MISSING(op_ret, op_errno) (op_ret == -1 && op_errno == ENOENT)
 
