@@ -94,7 +94,7 @@ mq_inode_loc_fill (const char *parent_gfid, inode_t *inode, loc_t *loc)
 
 ignore_parent:
         ret = inode_path (inode, NULL, &resolvedpath);
-        if (ret < 0)
+        if (ret <= 0)
                 goto err;
 
         ret = mq_loc_fill (loc, inode, parent, resolvedpath);
@@ -136,8 +136,8 @@ mq_get_contribution_node (inode_t *inode, quota_inode_ctx_t *ctx)
         inode_contribution_t    *contri = NULL;
         inode_contribution_t    *temp   = NULL;
 
-        GF_VALIDATE_OR_GOTO ("marker", inode, out);
-        GF_VALIDATE_OR_GOTO ("marker", ctx, out);
+        if (!inode || !ctx)
+                goto out;
 
         list_for_each_entry (temp, &ctx->contribution_head, contri_list) {
                 if (uuid_compare (temp->gfid, inode->gfid) == 0) {
