@@ -1,4 +1,4 @@
-/*Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
+/*Copyright (c) 2008-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
@@ -32,20 +32,16 @@
 #define QUOTA_DIRTY_KEY "trusted.glusterfs.quota.dirty"
 
 #define CONTRIBUTION "contri"
-#define VOL_NAME volname
 #define CONTRI_KEY_MAX 512
 #define READDIR_BUF 4096
 
-char volname [40];
-
 #define QUOTA_STACK_DESTROY(_frame, _this)              \
         do {                                            \
                 quota_local_t *_local = NULL;           \
                 _local = _frame->local;                 \
                 _frame->local = NULL;                   \
                 STACK_DESTROY (_frame->root);           \
-                quota_local_unref (_this, _local);      \
-                GF_FREE (_local);                       \
+                mq_local_unref (_this, _local);         \
         } while (0)
 
 
@@ -53,7 +49,7 @@ char volname [40];
         do {                                            \
                 ret = 0;                                \
                 var = GF_CALLOC (sizeof (type), 1,      \
-                                gf_marker_mt_##type);   \
+                                 gf_marker_mt_##type);  \
                 if (!var) {                             \
                         gf_log ("", GF_LOG_ERROR,       \
                                 "out of memory");       \
@@ -64,7 +60,7 @@ char volname [40];
 #define QUOTA_ALLOC_OR_GOTO(var, type, ret, label)      \
         do {                                            \
                 var = GF_CALLOC (sizeof (type), 1,      \
-                                gf_marker_mt_##type);   \
+                                 gf_marker_mt_##type);  \
                 if (!var) {                             \
                         gf_log ("", GF_LOG_ERROR,       \
                                 "out of memory");       \
@@ -74,42 +70,36 @@ char volname [40];
                 ret = 0;                                \
         } while (0);
 
-#define GET_CONTRI_KEY(var, _gfid, _ret)                                \
-        do {                                                            \
-                char    _gfid_unparsed[40];                             \
-                uuid_unparse (_gfid, _gfid_unparsed);                   \
+#define GET_CONTRI_KEY(var, _gfid, _ret)        \
+        do {                                    \
+                char _gfid_unparsed[40];        \
+                uuid_unparse (_gfid, _gfid_unparsed); \
                 _ret = snprintf (var, CONTRI_KEY_MAX, QUOTA_XATTR_PREFIX \
-                                ".%s.%s." CONTRIBUTION, VOL_NAME,       \
-                                _gfid_unparsed);                        \
+                                 ".%s.%s." CONTRIBUTION, "quota", \
+                                 _gfid_unparsed); \
         } while (0);
 
-#define QUOTA_SAFE_INCREMENT(lock, var)                 \
-        do {                                            \
-                LOCK (lock);                            \
-                        var ++;                         \
-                UNLOCK (lock);                          \
+#define QUOTA_SAFE_INCREMENT(lock, var)         \
+        do {                                    \
+                LOCK (lock);                    \
+                var ++;                         \
+                UNLOCK (lock);                  \
         } while (0)
 
-#define QUOTA_SAFE_DECREMENT(lock, var)                 \
-        do {                                            \
-                LOCK (lock);                            \
-                        var --;                         \
-                UNLOCK (lock);                          \
-        } while (0)
-
-
 struct quota_inode_ctx {
-        int64_t size;
-        int8_t  dirty;
-        gf_lock_t lock;
-        struct list_head  contribution_head;
+        int64_t                size;
+        int8_t                 dirty;
+        gf_boolean_t           updation_status;
+        gf_lock_t              lock;
+        struct list_head       contribution_head;
 };
 typedef struct quota_inode_ctx quota_inode_ctx_t;
 
 struct inode_contribution {
         struct list_head contri_list;
-        int64_t  contribution;
-        uuid_t   gfid;
+        int64_t          contribution;
+        uuid_t           gfid;
+  gf_lock_t lock;
 };
 typedef struct inode_contribution inode_contribution_t;
 
@@ -119,15 +109,16 @@ struct quota_local {
         int32_t err;
         int32_t ref;
         int64_t sum;
+        int64_t size;
         int32_t hl_count;
         int32_t dentry_child_count;
 
-        fd_t   *fd;
+        fd_t         *fd;
         call_frame_t *frame;
-        gf_lock_t lock;
+        gf_lock_t     lock;
 
-        loc_t   loc;
-        loc_t   parent_loc;
+        loc_t loc;
+        loc_t parent_loc;
 
         quota_inode_ctx_t    *ctx;
         inode_contribution_t *contri;
@@ -135,36 +126,36 @@ struct quota_local {
 typedef struct quota_local quota_local_t;
 
 int32_t
-get_lock_on_parent (call_frame_t *, xlator_t *);
+mq_get_lock_on_parent (call_frame_t *, xlator_t *);
 
 int32_t
-quota_req_xattr (xlator_t *, loc_t *, dict_t *);
+mq_req_xattr (xlator_t *, loc_t *, dict_t *);
 
 int32_t
 init_quota_priv (xlator_t *);
 
 int32_t
-quota_xattr_state (xlator_t *, loc_t *, dict_t *, struct iatt);
+mq_xattr_state (xlator_t *, loc_t *, dict_t *, struct iatt);
 
 int32_t
-quota_set_inode_xattr (xlator_t *, loc_t *);
+mq_set_inode_xattr (xlator_t *, loc_t *);
 
 int
-initiate_quota_txn (xlator_t *, loc_t *);
+mq_initiate_quota_txn (xlator_t *, loc_t *);
 
 int32_t
-quota_dirty_inode_readdir (call_frame_t *, void *, xlator_t *,
-                           int32_t, int32_t, fd_t *);
+mq_dirty_inode_readdir (call_frame_t *, void *, xlator_t *,
+                        int32_t, int32_t, fd_t *);
 
 int32_t
-reduce_parent_size (xlator_t *, loc_t *);
+mq_reduce_parent_size (xlator_t *, loc_t *, int64_t);
 
 int32_t
-quota_rename_update_newpath (xlator_t *, loc_t *);
+mq_rename_update_newpath (xlator_t *, loc_t *);
 
 int32_t
-inspect_file_xattr (xlator_t *this, loc_t *loc, dict_t *dict, struct iatt buf);
+mq_inspect_file_xattr (xlator_t *this, loc_t *loc, dict_t *dict, struct iatt buf);
 
 int32_t
-quota_forget (xlator_t *, quota_inode_ctx_t *);
+mq_forget (xlator_t *, quota_inode_ctx_t *);
 #endif
