@@ -431,12 +431,18 @@ marker_create_frame (xlator_t *this, marker_local_t *local)
 int32_t
 marker_xtime_update_marks (xlator_t *this, marker_local_t *local)
 {
+        GF_VALIDATE_OR_GOTO ("marker", this, out);
+        GF_VALIDATE_OR_GOTO (this->name, local, out);
+
+        if (local->pid == -1)
+                goto out;
+
         marker_gettimeofday (local);
 
         marker_local_ref (local);
 
         marker_create_frame (this, local);
-
+out:
         return 0;
 }
 
