@@ -1,3 +1,21 @@
+/*Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
+  This file is part of GlusterFS.
+
+  GlusterFS is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published
+  by the Free Software Foundation; either version 3 of the License,
+  or (at your option) any later version.
+
+  GlusterFS is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see
+  <http://www.gnu.org/licenses/>.
+*/
+
 #ifndef _LIBXLATOR_H
 #define _LIBXLATOR_H
 
@@ -56,10 +74,13 @@ struct marker_str {
         xlator_specf_unwind_t  xl_specf_unwind;
         void                  *xl_local;
         char                  *vol_uuid;
+        uint8_t                retval;
 };
 
+typedef struct marker_str xl_marker_local_t;
+
 static inline gf_boolean_t
-marker_has_volinfo (struct marker_str *marker)
+marker_has_volinfo (xl_marker_local_t *marker)
 {
        if (marker->volmark)
                 return _gf_true;
