@@ -256,9 +256,6 @@ glusterd_check_option_exists(char *optstring, char **completion);
 int
 set_xlator_option (dict_t *dict, char *key, char *value);
 
-char *
-glusterd_check_brick_rb_part (char *bricks, int count, glusterd_volinfo_t *volinfo);
-
 void
 glusterd_do_replace_brick (void *data);
 int
@@ -278,7 +275,6 @@ int32_t
 glusterd_handle_brick_rsp (glusterd_brickinfo_t *brickinfo,
                            glusterd_op_t op, dict_t *rsp_dict, dict_t *ctx_dict,
                            char **op_errstr);
-void glusterd_op_brick_disconnect (void *data);
 int32_t
 glusterd_op_init_ctx (glusterd_op_t op);
 int32_t
