@@ -69,6 +69,14 @@ migrate_xattrs_of_file (int src, int dst)
 
         while (size > size_processed) {
                 key = &list[size_processed];
+
+                /* SELinux setting happens from backend filesystem itself,
+                   don't try to copy it */
+                if (strcmp (key, "security.selinux") == 0) {
+                        size_processed += strlen (key) + 1;
+                        continue;
+                }
+
                 len = fgetxattr (src, key, value, 4096);
                 if (len < 0) {
                         gf_log (THIS->name, GF_LOG_ERROR,
@@ -197,6 +205,7 @@ gf_glusterd_rebalance_move_data (glusterd_volinfo_t *volinfo, const char *dir)
                                 full_path, strerror (errno));
                         close (dst_fd);
                         close (src_fd);
+                        unlink (tmp_filename);
                         continue;
                 }
 
@@ -221,6 +230,7 @@ gf_glusterd_rebalance_move_data (glusterd_volinfo_t *volinfo, const char *dir)
                                 full_path, strerror (errno));
                         close (dst_fd);
                         close (src_fd);
+                        unlink (tmp_filename);
                         continue;
                 }
 
@@ -231,6 +241,7 @@ gf_glusterd_rebalance_move_data (glusterd_volinfo_t *volinfo, const char *dir)
                                 tmp_filename, strerror (errno));
                         close (dst_fd);
                         close (src_fd);
+                        unlink (tmp_filename);
                         continue;
                 }
 
@@ -242,6 +253,7 @@ gf_glusterd_rebalance_move_data (glusterd_volinfo_t *volinfo, const char *dir)
                                 full_path);
                         close (dst_fd);
                         close (src_fd);
+                        unlink (tmp_filename);
                         continue;
                 }
 
@@ -251,6 +263,7 @@ gf_glusterd_rebalance_move_data (glusterd_volinfo_t *volinfo, const char *dir)
                                 full_path);
                         close (dst_fd);
                         close (src_fd);
+                        unlink (tmp_filename);
                         continue;
                 }
 
@@ -731,6 +744,17 @@ glusterd_rebalance_cmd_validate (int cmd, char *volname,
                           "be started to perform rebalance", volname);
                 goto out;
         }
+
+        if ((*volinfo)->brick_count <= (((*volinfo)->sub_count) ?
+                                        (*volinfo)->sub_count : 1)) {
+                /* distribute translator is not involved, no use of
+                   rebalancing */
+                snprintf (op_errstr, len, "nothing to rebalance on volume %s",
+                          volname);
+                gf_log (THIS->name, GF_LOG_WARNING, "%s", op_errstr);
+                goto out;
+        }
+
         ret = 0;
 out:
         gf_log ("glusterd", GF_LOG_DEBUG, "Returning %d", ret);
