@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2007-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU Affero General Public License as published
+  it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
+  General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License
+  You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -532,8 +532,8 @@ glusterd_volume_exclude_options_write (int fd, glusterd_volinfo_t *volinfo)
         GF_ASSERT (fd > 0);
         GF_ASSERT (volinfo);
 
-        char                    buf[4096] = {0,};
-        int32_t                 ret = -1;
+        char                    buf[PATH_MAX] = {0,};
+        int32_t                 ret           = -1;
 
         snprintf (buf, sizeof (buf), "%d", volinfo->type);
         ret = glusterd_store_save_value (fd, GLUSTERD_STORE_KEY_VOL_TYPE, buf);
@@ -633,6 +633,21 @@ out:
         return ret;
 }
 
+static void
+glusterd_store_rbstatepath_set (glusterd_volinfo_t *volinfo, char *rbstatepath,
+                                size_t len)
+{
+        char    voldirpath[PATH_MAX] = {0,};
+        GF_ASSERT (volinfo);
+        GF_ASSERT (rbstatepath);
+        GF_ASSERT (len >= PATH_MAX);
+
+        glusterd_store_voldirpath_set (volinfo, voldirpath,
+                                       sizeof (voldirpath));
+        snprintf (rbstatepath, len, "%s/%s", voldirpath,
+                  GLUSTERD_VOLUME_RBSTATE_FILE);
+}
+
 static void
 glusterd_store_volfpath_set (glusterd_volinfo_t *volinfo, char *volfpath,
                              size_t len)
@@ -647,6 +662,20 @@ glusterd_store_volfpath_set (glusterd_volinfo_t *volinfo, char *volfpath,
         snprintf (volfpath, len, "%s/%s", voldirpath, GLUSTERD_VOLUME_INFO_FILE);
 }
 
+int32_t
+glusterd_store_create_rbstate_shandle_on_absence (glusterd_volinfo_t *volinfo)
+{
+        char            rbstatepath[PATH_MAX] = {0};
+        int32_t         ret                   = 0;
+
+        GF_ASSERT (volinfo);
+
+        glusterd_store_rbstatepath_set (volinfo, rbstatepath, sizeof (rbstatepath));
+        ret = glusterd_store_handle_create_on_absence (&volinfo->rb_shandle,
+                                                       rbstatepath);
+        return ret;
+}
+
 int32_t
 glusterd_store_create_vol_shandle_on_absence (glusterd_volinfo_t *volinfo)
 {
@@ -682,6 +711,73 @@ out:
         return ret;
 }
 
+int32_t
+glusterd_store_rbstate_write (int fd, glusterd_volinfo_t *volinfo)
+{
+        int     ret             = -1;
+        char    buf[PATH_MAX]   = {0, };
+
+        GF_ASSERT (fd > 0);
+        GF_ASSERT (volinfo);
+
+        snprintf (buf, sizeof (buf), "%d", volinfo->rb_status);
+        ret = glusterd_store_save_value (fd, GLUSTERD_STORE_KEY_RB_STATUS,
+                                         buf);
+        if (ret)
+                goto out;
+
+        if (volinfo->rb_status > GF_RB_STATUS_NONE) {
+
+                snprintf (buf, sizeof (buf), "%s:%s",
+                          volinfo->src_brick->hostname,
+                          volinfo->src_brick->path);
+                ret = glusterd_store_save_value (fd, GLUSTERD_STORE_KEY_RB_SRC_BRICK,
+                                                 buf);
+                if (ret)
+                        goto out;
+
+                snprintf (buf, sizeof (buf), "%s:%s",
+                          volinfo->dst_brick->hostname,
+                          volinfo->dst_brick->path);
+                ret = glusterd_store_save_value (fd, GLUSTERD_STORE_KEY_RB_DST_BRICK,
+                                                 buf);
+                if (ret)
+                        goto out;
+        }
+
+out:
+        gf_log (THIS->name, GF_LOG_DEBUG, "Returning %d", ret);
+        return ret;
+}
+
+int32_t
+glusterd_store_perform_rbstate_store (glusterd_volinfo_t *volinfo)
+{
+        int                         fd = -1;
+        int32_t                     ret = -1;
+        GF_ASSERT (volinfo);
+
+        fd = glusterd_store_mkstemp (volinfo->rb_shandle);
+        if (fd <= 0) {
+                ret = -1;
+                goto out;
+        }
+
+        ret = glusterd_store_rbstate_write (fd, volinfo);
+        if (ret)
+                goto out;
+
+        ret = glusterd_store_rename_tmppath (volinfo->rb_shandle);
+
+out:
+        if (ret && (fd > 0))
+                glusterd_store_unlink_tmppath (volinfo->rb_shandle);
+        if (fd > 0)
+                close (fd);
+        gf_log ("", GF_LOG_DEBUG, "Returning %d", ret);
+        return ret;
+}
+
 int32_t
 glusterd_store_perform_volume_store (glusterd_volinfo_t *volinfo)
 {
@@ -744,9 +840,18 @@ glusterd_store_volinfo (glusterd_volinfo_t *volinfo, glusterd_volinfo_ver_ac_t a
         if (ret)
                 goto out;
 
+        ret = glusterd_store_create_rbstate_shandle_on_absence (volinfo);
+        if (ret)
+                goto out;
+
         ret = glusterd_store_perform_volume_store (volinfo);
         if (ret)
                 goto out;
+
+        ret = glusterd_store_perform_rbstate_store (volinfo);
+        if (ret)
+                goto out;
+
         //checksum should be computed at the end
         ret = glusterd_volume_compute_cksum (volinfo);
         if (ret)
@@ -838,17 +943,57 @@ out:
 }
 
 
+int
+glusterd_store_read_and_tokenize (FILE *file, char *str,
+                                  char **iter_key, char **iter_val,
+                                  glusterd_store_op_errno_t *store_errno)
+{
+        int32_t  ret = -1;
+
+        GF_ASSERT (file);
+        GF_ASSERT (str);
+        GF_ASSERT (iter_key);
+        GF_ASSERT (iter_val);
+        GF_ASSERT (store_errno);
+
+        ret = fscanf (file, "%s", str);
+        if (ret <= 0) {
+                ret = -1;
+                *store_errno = GD_STORE_EOF;
+                goto out;
+        }
+
+        *iter_key = strtok (str, "=");
+        if (*iter_key == NULL) {
+                ret = -1;
+                *store_errno = GD_STORE_KEY_NULL;
+                goto out;
+        }
+
+        *iter_val = strtok (NULL, "=");
+        if (*iter_key == NULL) {
+                ret = -1;
+                *store_errno = GD_STORE_VALUE_NULL;
+                goto out;
+        }
+
+        *store_errno = GD_STORE_SUCCESS;
+        ret = 0;
+out:
+        return ret;
+}
 
 int32_t
 glusterd_store_retrieve_value (glusterd_store_handle_t *handle,
                                char *key, char **value)
 {
         int32_t         ret = -1;
-        char            scan_str[4096] = {0,};
+        char            *scan_str = NULL;
         char            *iter_key = NULL;
         char            *iter_val = NULL;
-        char            *str = NULL;
         char            *free_str = NULL;
+        struct stat     st        = {0,};
+        glusterd_store_op_errno_t store_errno = GD_STORE_SUCCESS;
 
         GF_ASSERT (handle);
 
@@ -863,35 +1008,43 @@ glusterd_store_retrieve_value (glusterd_store_handle_t *handle,
                 goto out;
         }
 
-        ret = fscanf (handle->read, "%s", scan_str);
+        ret = fstat (handle->fd, &st);
+        if (ret < 0) {
+                gf_log ("glusterd", GF_LOG_WARNING,
+                        "stat on file failed");
+                ret = -1;
+                store_errno = GD_STORE_STAT_FAILED;
+                goto out;
+        }
 
-        while (ret != EOF) {
-                if (free_str) {
-                        GF_FREE (free_str);
-                        free_str = NULL;
-                }
-                str = gf_strdup (scan_str);
-                if (!str)
+        scan_str = GF_CALLOC (1, st.st_size,
+                              gf_gld_mt_char);
+        if (scan_str == NULL) {
+                ret = -1;
+                store_errno = GD_STORE_ENOMEM;
+                goto out;
+        }
+
+        free_str = scan_str;
+
+        do {
+                ret = glusterd_store_read_and_tokenize (handle->read, scan_str,
+                                                        &iter_key, &iter_val,
+                                                        &store_errno);
+                if (ret < 0) {
                         goto out;
-                else
-                        free_str = str;
-                iter_key = strtok (str, "=");
+                }
+
                 gf_log ("", GF_LOG_DEBUG, "key %s read", iter_key);
 
                 if (!strcmp (key, iter_key)) {
                         gf_log ("", GF_LOG_DEBUG, "key %s found", key);
-                        iter_val = strtok (NULL, "=");
                         ret = 0;
                         if (iter_val)
                                 *value = gf_strdup (iter_val);
                         goto out;
                 }
-
-                ret = fscanf (handle->read, "%s", scan_str);
-        }
-
-        if (EOF == ret)
-                ret = -1;
+        } while (1);
 out:
         if (handle->fd > 0) {
                 close (handle->fd);
@@ -908,16 +1061,20 @@ int32_t
 glusterd_store_save_value (int fd, char *key, char *value)
 {
         int32_t         ret = -1;
-        char            buf[4096] = {0,};
 
         GF_ASSERT (fd > 0);
         GF_ASSERT (key);
         GF_ASSERT (value);
 
-        snprintf (buf, sizeof (buf), "%s=%s\n", key, value);
-        ret = write (fd, buf, strlen (buf));
+        ret = write (fd, key, strlen (key));
+        if (ret > 0)
+                ret = write (fd, "=", strlen ("="));
+        if (ret > 0)
+                ret = write (fd, value, strlen (value));
+        if (ret > 0)
+                ret = write (fd, "\n", strlen ("\n"));
 
-        if (ret < 0) {
+        if (ret <= 0) {
                 gf_log ("", GF_LOG_CRITICAL, "Unable to store key: %s,"
                         "value: %s, error: %s", key, value,
                         strerror (errno));
@@ -979,13 +1136,16 @@ out:
 }
 
 int
-glusterd_store_handle_retrieve (char *path, glusterd_store_handle_t **handle)
+glusterd_store_handle_retrieve (char *path, glusterd_store_handle_t **handle,
+                                int *op_errno)
 {
         int32_t                 ret = -1;
         struct stat statbuf = {0};
 
         ret = stat (path, &statbuf);
         if (ret) {
+                if (op_errno)
+                        *op_errno = errno;
                 gf_log ("glusterd", GF_LOG_ERROR, "Unable to retrieve store "
                         "handle for %s, error: %s", path, strerror (errno));
                 goto out;
@@ -1084,7 +1244,7 @@ glusterd_retrieve_uuid ()
         if (!priv->handle) {
                 snprintf (path, PATH_MAX, "%s/%s", priv->workdir,
                           GLUSTERD_INFO_FILE);
-                ret = glusterd_store_handle_retrieve (path, &handle);
+                ret = glusterd_store_handle_retrieve (path, &handle, NULL);
 
                 if (ret) {
                         gf_log ("", GF_LOG_ERROR, "Unable to get store "
@@ -1199,11 +1359,11 @@ glusterd_store_iter_get_next (glusterd_store_iter_t *iter,
                               glusterd_store_op_errno_t *op_errno)
 {
         int32_t         ret = -1;
-        char            scan_str[4096] = {0,};
-        char            *str = NULL;
+        char            *scan_str = NULL;
         char            *free_str = NULL;
         char            *iter_key = NULL;
         char            *iter_val = NULL;
+        struct stat     st        = {0,};
         glusterd_store_op_errno_t store_errno = GD_STORE_SUCCESS;
 
         GF_ASSERT (iter);
@@ -1211,28 +1371,35 @@ glusterd_store_iter_get_next (glusterd_store_iter_t *iter,
         GF_ASSERT (key);
         GF_ASSERT (value);
 
-        *key = NULL;
-        *value = NULL;
-
-        ret = fscanf (iter->file, "%s", scan_str);
-
-        if (ret <= 0) {
+        ret = fstat (iter->fd, &st);
+        if (ret < 0) {
+                gf_log ("glusterd", GF_LOG_WARNING,
+                        "stat on file failed");
                 ret = -1;
-                store_errno = GD_STORE_EOF;
+                store_errno = GD_STORE_STAT_FAILED;
                 goto out;
         }
 
-        str = gf_strdup (scan_str);
-        if (!str) {
+        scan_str = GF_CALLOC (1, st.st_size,
+                              gf_gld_mt_char);
+        if (scan_str == NULL) {
                 ret = -1;
                 store_errno = GD_STORE_ENOMEM;
                 goto out;
-        } else {
-                free_str = str;
         }
 
-        iter_key = strtok (str, "=");
-        iter_val = strtok (NULL, "=");
+        *key = NULL;
+        *value = NULL;
+
+        free_str = scan_str;
+
+        ret = glusterd_store_read_and_tokenize (iter->file, scan_str,
+                                                &iter_key, &iter_val,
+                                                &store_errno);
+        if (ret < 0) {
+                goto out;
+        }
+
 
         ret = glusterd_store_validate_key_value (iter->filepath, iter_key,
                                                  iter_val, &store_errno);
@@ -1383,7 +1550,7 @@ glusterd_store_retrieve_bricks (glusterd_volinfo_t *volinfo)
 
                 tmpvalue = NULL;
 
-                ret = glusterd_store_handle_retrieve (path, &brickinfo->shandle);
+                ret = glusterd_store_handle_retrieve (path, &brickinfo->shandle, NULL);
 
                 if (ret)
                         goto out;
@@ -1460,19 +1627,104 @@ out:
 }
 
 
+int32_t
+glusterd_store_retrieve_rbstate (char   *volname, int *rb_errno)
+{
+        int32_t                   ret                   = -1;
+        glusterd_volinfo_t        *volinfo              = NULL;
+        glusterd_store_iter_t     *iter                 = NULL;
+        char                      *key                  = NULL;
+        char                      *value                = NULL;
+        char                      volpath[PATH_MAX]     = {0,};
+        glusterd_conf_t           *priv                 = NULL;
+        char                      path[PATH_MAX]        = {0,};
+        glusterd_store_op_errno_t op_errno              = GD_STORE_SUCCESS;
+
+        priv = THIS->private;
+
+        ret = glusterd_volinfo_find (volname, &volinfo);
+        if (ret) {
+                gf_log (THIS->name, GF_LOG_ERROR, "Couldn't get"
+                        "volinfo for %s.", volname);
+                goto out;
+        }
+
+        GLUSTERD_GET_VOLUME_DIR(volpath, volinfo, priv);
+        snprintf (path, sizeof (path), "%s/%s", volpath,
+                  GLUSTERD_VOLUME_RBSTATE_FILE);
+
+        ret = glusterd_store_handle_retrieve (path, &volinfo->rb_shandle, rb_errno);
+
+        if (ret)
+                goto out;
+
+        ret = glusterd_store_iter_new (volinfo->rb_shandle, &iter);
+
+        if (ret)
+                goto out;
+
+        ret = glusterd_store_iter_get_next (iter, &key, &value, &op_errno);
+        if (ret)
+                goto out;
+
+        while (!ret) {
+                if (!strncmp (key, GLUSTERD_STORE_KEY_RB_STATUS,
+                              strlen (GLUSTERD_STORE_KEY_RB_STATUS))) {
+                        volinfo->rb_status = atoi (value);
+                }
+
+                if (volinfo->rb_status > GF_RB_STATUS_NONE) {
+                        if (!strncmp (key, GLUSTERD_STORE_KEY_RB_SRC_BRICK,
+                                      strlen (GLUSTERD_STORE_KEY_RB_SRC_BRICK))) {
+                                ret = glusterd_brickinfo_from_brick (value,
+                                                                     &volinfo->src_brick);
+                                if (ret)
+                                        goto out;
+                        } else if (!strncmp (key, GLUSTERD_STORE_KEY_RB_DST_BRICK,
+                                             strlen (GLUSTERD_STORE_KEY_RB_DST_BRICK))) {
+                                ret = glusterd_brickinfo_from_brick (value,
+                                                                     &volinfo->dst_brick);
+                                if (ret)
+                                        goto out;
+                        }
+                }
+
+                GF_FREE (key);
+                GF_FREE (value);
+                key = NULL;
+                value = NULL;
+
+                ret = glusterd_store_iter_get_next (iter, &key, &value,
+                                                    &op_errno);
+        }
+
+        if (op_errno != GD_STORE_EOF)
+                goto out;
+
+        ret = glusterd_store_iter_destroy (iter);
+
+        if (ret)
+                goto out;
+
+out:
+        gf_log ("", GF_LOG_DEBUG, "Returning with %d", ret);
+
+        return ret;
+}
+
 int32_t
 glusterd_store_retrieve_volume (char    *volname)
 {
-        int32_t                 ret = -1;
-        glusterd_volinfo_t      *volinfo = NULL;
-        glusterd_store_iter_t   *iter = NULL;
-        char                    *key = NULL;
-        char                    *value = NULL;
-        char                    volpath[PATH_MAX] = {0,};
-        glusterd_conf_t         *priv = NULL;
-        char                    path[PATH_MAX] = {0,};
-        int                     exists = 0;
-        glusterd_store_op_errno_t op_errno = GD_STORE_SUCCESS;
+        int32_t                   ret                   = -1;
+        glusterd_volinfo_t        *volinfo              = NULL;
+        glusterd_store_iter_t     *iter                 = NULL;
+        char                      *key                  = NULL;
+        char                      *value                = NULL;
+        char                      volpath[PATH_MAX]     = {0,};
+        glusterd_conf_t           *priv                 = NULL;
+        char                      path[PATH_MAX]        = {0,};
+        int                       exists                = 0;
+        glusterd_store_op_errno_t op_errno              = GD_STORE_SUCCESS;
 
         ret = glusterd_volinfo_new (&volinfo);
 
@@ -1487,7 +1739,7 @@ glusterd_store_retrieve_volume (char    *volname)
         snprintf (path, sizeof (path), "%s/%s", volpath,
                   GLUSTERD_VOLUME_INFO_FILE);
 
-        ret = glusterd_store_handle_retrieve (path, &volinfo->shandle);
+        ret = glusterd_store_handle_retrieve (path, &volinfo->shandle, NULL);
 
         if (ret)
                 goto out;
@@ -1506,33 +1758,34 @@ glusterd_store_retrieve_volume (char    *volname)
                               strlen (GLUSTERD_STORE_KEY_VOL_TYPE))) {
                         volinfo->type = atoi (value);
                 } else if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_COUNT,
-                            strlen (GLUSTERD_STORE_KEY_VOL_COUNT))) {
+                                     strlen (GLUSTERD_STORE_KEY_VOL_COUNT))) {
                         volinfo->brick_count = atoi (value);
                 } else if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_STATUS,
-                            strlen (GLUSTERD_STORE_KEY_VOL_STATUS))) {
+                                     strlen (GLUSTERD_STORE_KEY_VOL_STATUS))) {
                         volinfo->status = atoi (value);
                 } else if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_VERSION,
-                            strlen (GLUSTERD_STORE_KEY_VOL_VERSION))) {
+                                     strlen (GLUSTERD_STORE_KEY_VOL_VERSION))) {
                         volinfo->version = atoi (value);
                 } else if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_PORT,
-                            strlen (GLUSTERD_STORE_KEY_VOL_PORT))) {
+                                     strlen (GLUSTERD_STORE_KEY_VOL_PORT))) {
                         volinfo->port = atoi (value);
                 } else if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_SUB_COUNT,
-                            strlen (GLUSTERD_STORE_KEY_VOL_SUB_COUNT))) {
+                                     strlen (GLUSTERD_STORE_KEY_VOL_SUB_COUNT))) {
                         volinfo->sub_count = atoi (value);
                 } else if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_TRANSPORT,
-                            strlen (GLUSTERD_STORE_KEY_VOL_TRANSPORT))) {
+                                     strlen (GLUSTERD_STORE_KEY_VOL_TRANSPORT))) {
                         volinfo->transport_type = atoi (value);
                         volinfo->nfs_transport_type = volinfo->transport_type;
                         if (volinfo->transport_type == GF_TRANSPORT_BOTH_TCP_RDMA) {
                                 volinfo->nfs_transport_type = GF_DEFAULT_NFS_TRANSPORT;
                         }
                 } else if (!strncmp (key, GLUSTERD_STORE_KEY_VOL_ID,
-                            strlen (GLUSTERD_STORE_KEY_VOL_ID))) {
+                                     strlen (GLUSTERD_STORE_KEY_VOL_ID))) {
                         ret = uuid_parse (value, volinfo->volume_id);
                         if (ret)
                                 gf_log ("", GF_LOG_WARNING,
                                         "failed to parse uuid");
+
                 } else if (strstr (key, "slave")) {
                         ret = dict_set_dynstr (volinfo->gsync_slaves, key,
                                                 gf_strdup (value));
@@ -1603,11 +1856,13 @@ out:
 int32_t
 glusterd_store_retrieve_volumes (xlator_t  *this)
 {
-        int32_t         ret = 0;
-        char            path[PATH_MAX] = {0,};
-        glusterd_conf_t *priv = NULL;
-        DIR             *dir = NULL;
-        struct dirent   *entry = NULL;
+        int32_t                ret              = 0;
+        char                   path[PATH_MAX]   = {0,};
+        glusterd_conf_t       *priv             = NULL;
+        DIR                   *dir              = NULL;
+        struct dirent         *entry            = NULL;
+        glusterd_volinfo_t    *volinfo          = NULL;
+        int                    rb_errno         = 0;
 
         GF_ASSERT (this);
         priv = this->private;
@@ -1634,6 +1889,22 @@ glusterd_store_retrieve_volumes (xlator_t  *this)
                                 "volume: %s", entry->d_name);
                         goto out;
                 }
+
+                ret = glusterd_store_retrieve_rbstate (entry->d_name,
+                                                       &rb_errno);
+                if (ret) {
+                        /* Backward compatibility */
+                        if (rb_errno == ENOENT) {
+                                gf_log ("", GF_LOG_INFO, "Creating a new rbstate "
+                                        "for volume: %s.", entry->d_name);
+                                ret = glusterd_volinfo_find (entry->d_name, &volinfo);
+                                ret = glusterd_store_create_rbstate_shandle_on_absence (volinfo);
+                                ret = glusterd_store_perform_rbstate_store (volinfo);
+                        } else {
+                                gf_log ("", GF_LOG_CRITICAL, "Unable to retrieve rbstate.");
+                                goto out;
+                        }
+                }
                 glusterd_for_each_entry (entry, dir);
         }
 
@@ -1928,7 +2199,7 @@ glusterd_store_retrieve_peers (xlator_t *this)
 
         while (entry) {
                 snprintf (filepath, PATH_MAX, "%s/%s", path, entry->d_name);
-                ret = glusterd_store_handle_retrieve (filepath, &shandle);
+                ret = glusterd_store_handle_retrieve (filepath, &shandle, NULL);
                 if (ret)
                         goto out;
 
