@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is GF_FREE software; you can redistribute it and/or modify
-  it under the terms of the GNU Affero General Public License as published
+  it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
+  General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License
+  You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -366,6 +366,14 @@ glusterd_crt_georep_folders (char *georepdir, glusterd_conf_t *conf)
 
 static int
 configure_syncdaemon (glusterd_conf_t *conf)
+#define libc_system system
+#define RUN_GSYNCD_CMD(prf) do {                                                \
+        ret = prf##_system (cmd);                                               \
+        if (ret) {                                                              \
+                gf_log ("", GF_LOG_ERROR, "command failed: \"%s\"", cmd);       \
+                goto out;                                                       \
+        }                                                                       \
+} while (0)
 {
         int ret = 0;
 #if SYNCDAEMON_COMPILE
@@ -409,66 +417,48 @@ configure_syncdaemon (glusterd_conf_t *conf)
 
         /* remote-gsyncd */
         strcpy (cmd + blen, "remote-gsyncd " GSYNCD_PREFIX"/gsyncd . .");
-        ret = system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(libc);
 
         strcpy (cmd + blen,
                 "remote-gsyncd /usr/local/libexec/glusterfs/gsyncd . ^ssh:");
-        ret = system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(libc);
 
         /* gluster-command */
         /* XXX $sbindir should be used (throughout the codebase) */
         strcpy (cmd + blen,
                 "gluster-command '"GFS_PREFIX"/sbin/glusterfs "
                 "--xlator-option *-dht.assert-no-child-down=true' . .");
-        ret = system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(libc);
 
         /* ssh-command */
         sprintf (cmd + blen,
                  "ssh-command "
                  "'ssh -oPasswordAuthentication=no -oStrictHostKeyChecking=no "
                  "-i %s/secret.pem' . .", georepdir);
-        ret = system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(libc);
 
         /* session-owner */
         uuid_unparse (conf->uuid, volid);
         sprintf (cmd + blen, "session-owner %s . .", volid);
-        ret = system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(libc);
 
         /* pid-file */
         sprintf (cmd + blen, "pid-file %s/${mastervol}/${eSlave}.pid . .", georepdir);
-        ret = gf_system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(gf);
 
         /* state-file */
         sprintf (cmd + blen, "state-file %s/${mastervol}/${eSlave}.status . .", georepdir);
-        ret = gf_system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(gf);
 
         /* log-file */
         strcpy (cmd + blen,
                 "log-file "DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"/${mastervol}/${eSlave}.log . .");
-        ret = gf_system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(gf);
 
         /* gluster-log-file */
         strcpy (cmd + blen, "gluster-log-file "
                 DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"/${mastervol}/${eSlave}.gluster.log . .");
-        ret = gf_system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(gf);
 
         /************
          * slave pre-configuration
@@ -478,23 +468,17 @@ configure_syncdaemon (glusterd_conf_t *conf)
         strcpy (cmd + blen,
                 "gluster-command '"GFS_PREFIX"/sbin/glusterfs "
                 "--xlator-option *-dht.assert-no-child-down=true' .");
-        ret = system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(libc);
 
         /* log-file */
         strcpy (cmd + blen,
                 "log-file "DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"-slaves/${session_owner}:${eSlave}.log .");
-        ret = gf_system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(gf);
 
         /* gluster-log-file */
         strcpy (cmd + blen, "gluster-log-file "
                 DEFAULT_LOG_FILE_DIRECTORY"/"GEOREP"-slaves/${session_owner}:${eSlave}.gluster.log .");
-        ret = gf_system (cmd);
-        if (ret)
-                goto out;
+        RUN_GSYNCD_CMD(gf);
 
  out:
 #else
@@ -502,6 +486,8 @@ configure_syncdaemon (glusterd_conf_t *conf)
 #endif
         return ret ? -1 : 0;
 }
+#undef libc_system
+#undef RUN_GSYNCD_CMD
 
 
 /*
