@@ -346,25 +346,15 @@ nfs3_stat_to_fattr3 (struct iatt *buf)
 
         fa.fsid = buf->ia_dev;
         fa.fileid = nfs3_iatt_gfid_to_ino (buf);
-        /* FIXME: Handle time resolutions for sub-second granularity */
-        if (buf->ia_atime == 9669) {
-                fa.mtime.seconds = 0;
-                fa.mtime.nseconds = 0;
-                fa.atime.seconds = 0;
-                fa.atime.nseconds = 0;
-        } else {
-                fa.mtime.seconds = buf->ia_mtime;
-                fa.mtime.nseconds = 0;
-                fa.atime.seconds = buf->ia_atime;
-                fa.atime.seconds = 0;
-                fa.atime.nseconds = 0;
-        }
 
         fa.atime.seconds = buf->ia_atime;
-        fa.atime.nseconds = 0;
+        fa.atime.nseconds = buf->ia_atime_nsec;
 
         fa.ctime.seconds = buf->ia_ctime;
-        fa.ctime.nseconds = 0;
+        fa.ctime.nseconds = buf->ia_ctime_nsec;
+
+        fa.mtime.seconds = buf->ia_mtime;
+        fa.mtime.nseconds = buf->ia_mtime_nsec;
 
         return fa;
 }
@@ -408,11 +398,10 @@ nfs3_stat_to_pre_op_attr (struct iatt *pre)
 
         poa.attributes_follow = TRUE;
         poa.pre_op_attr_u.attributes.size = pre->ia_size;
-        if (pre->ia_atime == 9669)
-                poa.pre_op_attr_u.attributes.mtime.seconds = 0;
-        else
-                poa.pre_op_attr_u.attributes.mtime.seconds = pre->ia_mtime;
+        poa.pre_op_attr_u.attributes.mtime.seconds = pre->ia_mtime;
+        poa.pre_op_attr_u.attributes.mtime.nseconds = pre->ia_mtime_nsec;
         poa.pre_op_attr_u.attributes.ctime.seconds = pre->ia_ctime;
+        poa.pre_op_attr_u.attributes.ctime.nseconds = pre->ia_ctime_nsec;
 
 out:
         return poa;
@@ -578,7 +567,7 @@ nfs3_accessbits (int32_t accbits)
                 accresult |= ACCESS3_READ;
 
         if (accbits & POSIX_WRITE)
-                accresult |= (ACCESS3_MODIFY | ACCESS3_EXTEND);
+                accresult |= (ACCESS3_MODIFY | ACCESS3_EXTEND | ACCESS3_DELETE);
 
         /* lookup on directory allowed only in case of execute permission */
         if (accbits & POSIX_EXEC)
@@ -595,7 +584,7 @@ nfs3_request_to_accessbits (int32_t accbits)
         if (accbits & ACCESS3_READ)
                 acc_request |= POSIX_READ;
 
-        if (accbits & (ACCESS3_MODIFY | ACCESS3_EXTEND))
+        if (accbits & (ACCESS3_MODIFY | ACCESS3_EXTEND | ACCESS3_DELETE))
                 acc_request |= POSIX_WRITE;
 
         /* For lookup on directory check for execute permission */
@@ -2514,6 +2503,14 @@ nfs3_fh_resolve_readdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                              int32_t op_ret, int32_t op_errno,
                              gf_dirent_t *entries);
 
+void
+nfs3_fh_resolve_not_found (nfs3_call_state_t *cs)
+{
+        cs->resolve_ret = -1;
+        cs->resolve_errno = ENOENT;
+        nfs3_call_resume (cs);
+}
+
 int
 nfs3_fh_resolve_found_entry (nfs3_call_state_t *cs, gf_dirent_t *candidate)
 {
@@ -2824,7 +2821,6 @@ int
 nfs3_fh_resolve_check_response (nfs3_call_state_t *cs)
 {
         int             ret = -EFAULT;
-        nfs_user_t      nfu = {0, };
         int             response = GF_NFS3_FHRESOLVE_NOTFOUND;
 
         if (!cs)
@@ -2845,10 +2841,7 @@ nfs3_fh_resolve_check_response (nfs3_call_state_t *cs)
                 break;
 
         case GF_NFS3_FHRESOLVE_NOTFOUND:
-                nfs_user_root_create (&nfu);
-                nfs_readdirp (cs->nfsx, cs->vol, &nfu, cs->resolve_dir_fd,
-                              GF_NFS3_DTPREF, cs->lastentryoffset,
-                              nfs3_fh_resolve_readdir_cbk, cs);
+                nfs3_fh_resolve_not_found (cs);
                 break;
         }
 
@@ -2920,8 +2913,9 @@ err:
 int
 nfs3_fh_resolve_inode_hard (nfs3_call_state_t *cs)
 {
-        int             ret = -EFAULT;
-        nfs_user_t      nfu = {0, };
+        int                     ret = -EFAULT;
+        nfs_user_t              nfu = {0, };
+        struct nfs_state        *nfs = NULL;
 
         if (!cs)
                 return ret;
@@ -2940,6 +2934,16 @@ nfs3_fh_resolve_inode_hard (nfs3_call_state_t *cs)
                 ", hashcount: %d, current hashidx %d",
                 uuid_utoa (cs->resolvefh.gfid),
                 cs->resolvefh.hashcount, cs->hashidx);
+
+        nfs = THIS->private;
+
+        LOCK (&nfs->lock);
+        {
+                nfs->res_stat.ino_cnt++;
+                nfs->res_stat.intvl_ino_cnt++;
+        }
+        UNLOCK (&nfs->lock);
+
         ret = nfs_root_loc_fill (cs->vol->itable, &cs->resolvedloc);
 
         if (ret == 0) {
@@ -2962,8 +2966,9 @@ out:
 int
 nfs3_fh_resolve_entry_hard (nfs3_call_state_t *cs)
 {
-        int             ret = -EFAULT;
-        nfs_user_t      nfu = {0, };
+        int                     ret = -EFAULT;
+        nfs_user_t              nfu = {0, };
+        struct nfs_state        *nfs = NULL;
 
         if (!cs)
                 return ret;
@@ -2973,6 +2978,14 @@ nfs3_fh_resolve_entry_hard (nfs3_call_state_t *cs)
         gf_log (GF_NFS3, GF_LOG_TRACE, "FH hard resolution: gfid: %s "
                 ", entry: %s, hashidx: %d", uuid_utoa (cs->resolvefh.gfid),
                 cs->resolventry, cs->hashidx);
+        nfs = THIS->private;
+
+        LOCK (&nfs->lock);
+        {
+                nfs->res_stat.entry_cnt++;
+                nfs->res_stat.intvl_entry_cnt++;
+        }
+        UNLOCK (&nfs->lock);
 
         ret = nfs_entry_loc_fill (cs->vol->itable, cs->resolvefh.gfid,
                                   cs->resolventry, &cs->resolvedloc,
