@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2007-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU Affero General Public License as published
+  it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
+  General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License
+  You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -36,6 +36,7 @@ void *
 str_to_ptr (char *string)
 {
         void *ptr = NULL;
+
         GF_VALIDATE_OR_GOTO ("io-cache", string, out);
 
         ptr = (void *)strtoul (string, NULL, 16);
@@ -122,9 +123,10 @@ ioc_inode_wakeup (call_frame_t *frame, ioc_inode_t *ioc_inode,
                                 ioc_inode_lock (ioc_inode);
                                 {
                                         page_waitq =
-                                                ioc_page_wakeup (waiter_page);
+                                                __ioc_page_wakeup (waiter_page);
                                 }
                                 ioc_inode_unlock (ioc_inode);
+
                                 if (page_waitq)
                                         ioc_waitq_return (page_waitq);
                         } else {
@@ -132,17 +134,19 @@ ioc_inode_wakeup (call_frame_t *frame, ioc_inode_t *ioc_inode,
                                  * page->ready = 0, to avoid double faults
                                  */
                                 ioc_inode_lock (ioc_inode);
-
-                                if (waiter_page->ready) {
-                                        waiter_page->ready = 0;
-                                        need_fault = 1;
-                                } else {
-                                        gf_log (frame->this->name, GF_LOG_TRACE,
-                                                "validate frame(%p) is waiting"
-                                                "for in-transit page = %p",
-                                                frame, waiter_page);
+                                {
+                                        if (waiter_page->ready) {
+                                                waiter_page->ready = 0;
+                                                need_fault = 1;
+                                        } else {
+                                                gf_log (frame->this->name,
+                                                        GF_LOG_TRACE,
+                                                        "validate frame(%p) is "
+                                                        "waiting for "
+                                                        "in-transit page = %p",
+                                                        frame, waiter_page);
+                                        }
                                 }
-
                                 ioc_inode_unlock (ioc_inode);
 
                                 if (need_fault) {
