@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU Affero General Public License as published
+  it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
+  General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License
+  You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -36,7 +36,7 @@
 void *iot_worker (void *arg);
 int iot_workers_scale (iot_conf_t *conf);
 int __iot_workers_scale (iot_conf_t *conf);
-
+struct volume_options options[];
 
 call_stub_t *
 __iot_dequeue (iot_conf_t *conf)
@@ -2157,11 +2157,12 @@ int
 init (xlator_t *this)
 {
         iot_conf_t      *conf = NULL;
-        dict_t          *options = this->options;
+        dict_t          *xl_options = this->options;
         int              thread_count = IOT_DEFAULT_THREADS;
         int              idle_time = IOT_DEFAULT_IDLE;
         int              ret = -1;
         int              i = 0;
+        char            *def_val = NULL;
 
 	if (!this->children || this->children->next) {
 		gf_log ("io-threads", GF_LOG_ERROR,
@@ -2182,12 +2183,37 @@ init (xlator_t *this)
                 goto out;
         }
 
+        if ((ret = pthread_cond_init(&conf->cond, NULL)) != 0) {
+                gf_log (this->name, GF_LOG_ERROR,
+                        "pthread_cond_init failed (%d)", ret);
+                goto out;
+        }
+
+        if ((ret = pthread_mutex_init(&conf->mutex, NULL)) != 0) {
+                gf_log (this->name, GF_LOG_ERROR,
+                        "pthread_mutex_init failed (%d)", ret);
+                goto out;
+        }
+
         set_stack_size (conf);
 
-        thread_count = IOT_DEFAULT_THREADS;
+        if (xlator_get_volopt_info (&this->volume_options, "thread-count",
+                                    &def_val, NULL)) {
+                gf_log (this->name, GF_LOG_ERROR, "Default value of "
+                         "thread-count not found");
+                ret = -1;
+                goto out;
+        } else {
+                if (gf_string2int32 (def_val, &conf->max_count)) {
+                        gf_log (this->name, GF_LOG_ERROR, "Default value of "
+                                 "thread corrupt");
+                        ret = -1;
+                        goto out;
+                }
+        }
 
-	if (dict_get (options, "thread-count")) {
-                thread_count = data_to_int32 (dict_get (options,
+	if (dict_get (xl_options, "thread-count")) {
+                thread_count = data_to_int32 (dict_get (xl_options,
                                                         "thread-count"));
                 if (thread_count < IOT_MIN_THREADS) {
                         gf_log ("io-threads", GF_LOG_WARNING,
@@ -2204,8 +2230,8 @@ init (xlator_t *this)
         }
         conf->max_count = thread_count;
 
-	if (dict_get (options, "idle-time")) {
-                idle_time = data_to_int32 (dict_get (options,
+	if (dict_get (xl_options, "idle-time")) {
+                idle_time = data_to_int32 (dict_get (xl_options,
                                                      "idle-time"));
                 if (idle_time < 0)
                         idle_time = 1;
@@ -2292,7 +2318,11 @@ struct volume_options options[] = {
 	{ .key  = {"thread-count"},
 	  .type = GF_OPTION_TYPE_INT,
 	  .min  = IOT_MIN_THREADS,
-	  .max  = IOT_MAX_THREADS
+	  .max  = IOT_MAX_THREADS,
+          .default_value = "16",
+          .description = "Number of threads in IO threads translator which "
+                         "perform concurrent IO operations"
+
 	},
         {.key   = {"idle-time"},
          .type  = GF_OPTION_TYPE_INT,
