@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU Affero General Public License as published
+  it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
+  General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License
+  You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -345,6 +345,7 @@ server_setvolume (rpcsvc_request_t *req)
         dict_t              *params        = NULL;
         char                *name          = NULL;
         char                *process_uuid  = NULL;
+        char                *clnt_version  = NULL;
         xlator_t            *xl            = NULL;
         char                *msg           = NULL;
         char                *volfile_key   = NULL;
@@ -525,13 +526,20 @@ server_setvolume (rpcsvc_request_t *req)
                         "Authentication module not initialized");
         }
 
+        ret = dict_get_str (params, "client-version", &clnt_version);
+        if (ret)
+                gf_log (this->name, GF_LOG_INFO, "client-version not set, "
+                        "may be of older version");
+
         ret = gf_authenticate (params, config_params,
                                conf->auth_modules);
 
         if (ret == AUTH_ACCEPT) {
+
                 gf_log (this->name, GF_LOG_INFO,
-                        "accepted client from %s",
-                        (peerinfo)?peerinfo->identifier:"");
+                        "accepted client from %s (version: %s)",
+                        (peerinfo) ? peerinfo->identifier : "<>",
+                        (clnt_version) ? clnt_version : "old");
                 op_ret = 0;
                 conn->bound_xl = xl;
                 ret = dict_set_str (reply, "ERROR", "Success");
@@ -540,8 +548,10 @@ server_setvolume (rpcsvc_request_t *req)
                                 "failed to set error msg");
         } else {
                 gf_log (this->name, GF_LOG_ERROR,
-                        "Cannot authenticate client from %s",
-                        (peerinfo)? peerinfo->identifier:"<>");
+                        "Cannot authenticate client from %s %s",
+                        (peerinfo) ? peerinfo->identifier : "<>",
+                        (clnt_version) ? clnt_version : "old");
+
                 op_ret = -1;
                 op_errno = EACCES;
                 ret = dict_set_str (reply, "ERROR", "Authentication failed");
