@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2010-2011 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU Affero General Public License as published
+  it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Affero General Public License for more details.
+  General Public License for more details.
 
-  You should have received a copy of the GNU Affero General Public License
+  You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -794,7 +794,6 @@ server_connection_t *
 server_connection_get (xlator_t *this, const char *id)
 {
         server_connection_t *conn = NULL;
-        server_connection_t *trav = NULL;
         server_conf_t       *conf = NULL;
 
         GF_VALIDATE_OR_GOTO ("server", this, out);
@@ -804,27 +803,18 @@ server_connection_get (xlator_t *this, const char *id)
 
         pthread_mutex_lock (&conf->mutex);
         {
-                list_for_each_entry (trav, &conf->conns, list) {
-                        if (!strcmp (id, trav->id)) {
-                                conn = trav;
-                                break;
-                        }
-                }
+                conn = (void *) GF_CALLOC (1, sizeof (*conn),
+                                           gf_server_mt_conn_t);
+                if (!conn)
+                        goto unlock;
 
-                if (!conn) {
-                        conn = (void *) GF_CALLOC (1, sizeof (*conn),
-                                                   gf_server_mt_conn_t);
-                        if (!conn)
-                                goto unlock;
+                conn->id = gf_strdup (id);
+                conn->fdtable = gf_fd_fdtable_alloc ();
+                conn->ltable  = gf_lock_table_new ();
+                conn->this    = this;
+                pthread_mutex_init (&conn->lock, NULL);
 
-                        conn->id = gf_strdup (id);
-                        conn->fdtable = gf_fd_fdtable_alloc ();
-                        conn->ltable  = gf_lock_table_new ();
-                        conn->this    = this;
-                        pthread_mutex_init (&conn->lock, NULL);
-
-                        list_add (&conn->list, &conf->conns);
-                }
+                list_add (&conn->list, &conf->conns);
 
                 conn->ref++;
                 conn->active_transports++;
